package com.neurosdk2;

import static com.neurosdk2.Utils.ConvertReactToIntArray;
import static com.neurosdk2.Utils.ConvertSignalChannelsData;
import static com.neurosdk2.Utils.DATA_ID;
import static com.neurosdk2.Utils.GUID_ID;
import static com.neurosdk2.Utils.sendEvent;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neurosdk2.neuro.NeuroEEG;
import com.neurosdk2.neuro.Sensor;
import com.neurosdk2.neuro.types.EEGChannelInfo;
import com.neurosdk2.neuro.types.NeuroEEGAmplifierParam;
import com.neurosdk2.neuro.types.ResistChannelsData;
import com.neurosdk2.neuro.types.NeuroEEGFSStatus;
import com.neurosdk2.neuro.types.SensorCommand;
import com.neurosdk2.neuro.types.SensorFSStatus;
import com.neurosdk2.neuro.types.SignalChannelsData;

import static com.neurosdk2.Utils.ConvertByteArrayToStr;
import java.nio.charset.StandardCharsets;
import android.util.Base64;
import com.neurosdk2.neuro.types.SensorDiskInfo;
import com.neurosdk2.neuro.types.SensorFileData;
import com.neurosdk2.neuro.types.SensorFileInfo;


public class NeuroEEGImpl extends AFullSensor{

  public NeuroEEGImpl(Sensor sensor, String guid, ReactApplicationContext context){
    this.sensor = sensor;
    this.guid = guid;
    this.reactContext = context;
    // NeuroEEGFSStatus fsStatus = ((NeuroEEG)sensor).getFSStatus();
    // if(fsStatus.getStatus() == SensorFSStatus.OK){
    //   sensor.execCommand(SensorCommand.CommandFileSystemStreamClose);
    //   sensor.execCommand(SensorCommand.CommandFileSystemDisable);
    // }
    addCallbacks();
  }

  @Override
  protected void addCallbacks() {
    super.addCallbacks();
    ((NeuroEEG)sensor).ampModeChanged = (data)->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableMap map = Arguments.createMap();
      result.putInt(DATA_ID, data.index());
      sendEvent(reactContext,Constants.AMPMODE_CHANGED_EVENT_NAME.raw(), result);
    };
    ((NeuroEEG)sensor).signalDataReceived = data -> {
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableArray signalArray = Arguments.createArray();
      for (SignalChannelsData signal : data) {
        WritableMap map = Arguments.createMap();
        ConvertSignalChannelsData(signal, map);
        signalArray.pushMap(map);
      }
      result.putArray(DATA_ID, signalArray);
      sendEvent(reactContext,Constants.NEURRO_EEG_SIGNAL_DATA_EVENT_NAME.raw(), result);

    };
    ((NeuroEEG)sensor).resistDataReceived = data ->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableArray resistArray = Arguments.createArray();
      for (ResistChannelsData resist : data) {
        WritableMap map = Arguments.createMap();
        ConvertResistChannelsData(resist, map);
        resistArray.pushMap(map);
      }
      result.putArray(DATA_ID, resistArray);
      sendEvent(reactContext,Constants.NEURRO_EEG_RESIST_DATA_EVENT_NAME.raw(), result);
    };
    ((NeuroEEG)sensor).signalResistReceived = (signalData, resistData) ->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableArray signalArray = Arguments.createArray();
      for (SignalChannelsData signal : signalData) {
        WritableMap map = Arguments.createMap();
        ConvertSignalChannelsData(signal, map);
        signalArray.pushMap(map);
      }
      WritableArray resistArray = Arguments.createArray();
      for (ResistChannelsData resist : resistData) {
        WritableMap map = Arguments.createMap();
        ConvertResistChannelsData(resist, map);
        resistArray.pushMap(map);
      }
      WritableMap map = Arguments.createMap();
      map.putArray("signalData", signalArray);
      map.putArray("resistData", resistArray);
      result.putMap(DATA_ID, map);
      sendEvent(reactContext,Constants.NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME.raw(), result);
    };

    ((NeuroEEG)sensor).signalRawDataReceived = data ->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      int[] intData = new int[data.length];
      for (int i = 0; i < data.length; intData[i] = data[i++]);
      ReadableArray array = Arguments.fromArray(intData);
      result.putArray(DATA_ID, array);
      sendEvent(reactContext,Constants.NEURRO_EEG_RAW_DATA_EVENT_NAME.raw(), result);
    };
    ((NeuroEEG)sensor).fileStreamDataReceived = data ->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableArray array = Arguments.createArray();
      for (SensorFileData fileData : data) {
        WritableMap map = Arguments.createMap();
        map.putInt("OffsetStart", fileData.getOffsetStart());
        map.putInt("DataAmount", fileData.getDataAmount());
        int[] intData = new int[fileData.getData().length];
        for (int i = 0; i < fileData.getData().length; intData[i] = fileData.getData()[i++]);
        ReadableArray dataArray = Arguments.fromArray(intData);
        map.putArray("Data", dataArray);
        array.pushMap(map);
      }
      result.putArray(DATA_ID, array);
      sendEvent(reactContext,Constants.NEURRO_EEG_FILE_STREAM_DATA_EVENT_NAME.raw(), result);
    };

  }

  @Override
  protected void removeCallbacks() {
    super.removeCallbacks();
    ((NeuroEEG)sensor).ampModeChanged = null;
    ((NeuroEEG)sensor).signalDataReceived = null;
    ((NeuroEEG)sensor).resistDataReceived = null;
    ((NeuroEEG)sensor).signalResistReceived = null;

    ((NeuroEEG)sensor).signalRawDataReceived = null;
    ((NeuroEEG)sensor).fileStreamDataReceived = null;

  }

  @Override
  protected String getSensorName() {
    return "NeuroEEG";
  }

  @Override
  public int getSurveyId() throws Exception {
    return ((NeuroEEG)sensor).getSurveyId();
  }

  @Override
  public void setSurveyId(int id) throws Exception {
    ((NeuroEEG)sensor).setSurveyId(id);
  }

  @Override
  public int getAmpMode() throws Exception {
    return ((NeuroEEG)sensor).getAmpMode().index();
  }

  @Override
  public void getSupportedChannels(WritableArray channels) throws Exception {
    List<EEGChannelInfo> channelsList = ((NeuroEEG)sensor).getSupportedChannels();
    for (EEGChannelInfo channel : channelsList) {
      WritableMap channelMap = Arguments.createMap();
      channelMap.putInt("Id",channel.getId().index());
      channelMap.putInt("ChType",channel.getChType().index());
      channelMap.putString("Name",channel.getName());
      channelMap.putInt("Num",channel.getNum());
      channels.pushMap(channelMap);
    }
  }

  @Override
  public void getAmplifierParam(WritableMap map) throws Exception {
    NeuroEEGAmplifierParam param = ((NeuroEEG)sensor).getAmplifierParam();
    map.putBoolean("ReferentResistMesureAllow",param.getReferentResistMesureAllow());
    map.putInt("Frequency",param.getFrequency().index());
    map.putInt("ReferentMode",param.getReferentMode().index());
    map.putArray("ChannelMode",Arguments.fromArray(param.getRawChannelMode()));
    map.putArray("ChannelGain",Arguments.fromArray(param.getRawChannelGain()));
  }

  @Override
  public void setAmplifierParam(ReadableMap param) throws Exception {
    ((NeuroEEG)sensor).setAmplifierParam(new NeuroEEGAmplifierParam(
      param.getBoolean("ReferentResistMesureAllow"),
      (int) param.getInt("Frequency"),
      (int) param.getInt("ReferentMode"),
      ConvertReactToIntArray(param.getArray("ChannelMode")),
      ConvertReactToIntArray(param.getArray("ChannelGain"))
    ));
  }

  @Override
  public int getSamplingFrequencyResist() throws Exception {
    return ((NeuroEEG)sensor).getSamplingFrequencyResist().index();
  }

  @Override
  public void getFSStatus(WritableMap map) throws Exception {
    NeuroEEGFSStatus status = ((NeuroEEG)sensor).getFSStatus();
    map.putInt("Status",status.getStatus().index());
    map.putInt("IOStatus",status.getIOStatus().index());
    map.putInt("StreamStatus",status.getStreamStatus().index());
    map.putBoolean("AutosaveSignal",status.getAutosaveSignal());
  }

  @Override
  public void getFSDiskInfo(WritableMap map) throws Exception {
    SensorDiskInfo info = ((NeuroEEG)sensor).getFSDiskInfo();
    map.putDouble("TotalSize",info.getTotalSize());
    map.putDouble("FreeSize",info.getFreeSize());
  }

  @Override
  public void readFileInfo(String fileName, WritableMap fileInfo) throws Exception {
    SensorFileInfo info = ((NeuroEEG)sensor).readFileInfo(fileName);
    ConvertSensorFileInfo(info, fileInfo);
  }

  @Override
  public void readFileInfoAll(int maxFiles, WritableArray infoArray) throws Exception {
    List<SensorFileInfo> infos = ((NeuroEEG)sensor).readFileInfoAll(maxFiles);
      for (SensorFileInfo info : infos) {
      WritableMap infoMap = Arguments.createMap();
      ConvertSensorFileInfo(info, infoMap);
      infoArray.pushMap(infoMap);
    }
  }

  @Override
  public void writeFile(String fileName, ReadableArray data, int offset) throws Exception {
    // byte[] byteData = ConvertStrToByteArray(data);
    byte[] byteData = new byte[data.size()];
    for (int i = 0; i < data.size(); i++)
    {
      byteData[i] = (byte)data.getInt(i);
    }
    ((NeuroEEG)sensor).writeFile(fileName, byteData, offset);
  }

  @Override
  public String readFile(String fileName, int sizeData, int offset) throws Exception {
    byte[] arr = ((NeuroEEG)sensor).readFile(fileName, sizeData, offset);
    return ConvertByteArrayToStr(arr);
  }

  @Override
  public void deleteFile(String fileName) throws Exception {
    ((NeuroEEG)sensor).deleteFile(fileName);
  }

  @Override
  public void deleteAllFiles(String fileExt) throws Exception {
    ((NeuroEEG)sensor).deleteAllFiles(fileExt);
  }

  @Override
  public int readFileCRC32(String fileName, int totalSize, int offset) throws Exception {
    return ((NeuroEEG)sensor).readFileCRC32(fileName, totalSize, offset);
  }

  @Override
  public void fileStreamAutosave(String fileName) throws Exception {
    ((NeuroEEG)sensor).fileStreamAutosave(fileName);
  }

  @Override
  public void fileStreamRead(String fileName, int totalSize, int offset) throws Exception {
    ((NeuroEEG)sensor).fileStreamRead(fileName, totalSize, offset);
  }

  private static void ConvertSensorFileInfo(SensorFileInfo value, WritableMap result) {
    result.putString("FileName", value.getFileName());
    result.putInt("FileSize", value.getFileSize());
    result.putInt("ModifiedYear", value.getModifiedYear());
    result.putInt("ModifiedMonth", value.getModifiedMonth());
    result.putInt("ModifiedDayOfMonth", value.getModifiedDayOfMonth());
    result.putInt("ModifiedHour", value.getModifiedHour());
    result.putInt("ModifiedMin", value.getModifiedMin());
    result.putInt("ModifiedSec", value.getModifiedSec());
    result.putInt("Attribute", value.getAttribute());
  }

  private static byte[] ConvertReactToByteArray(ReadableArray array)
  {
    byte[] result = new byte[array.size()];
    ArrayList arrayList = Arguments.toList(array);
    for (int i = 0; i < result.length; i++) {
      result[i] = (byte)arrayList.get(i);
    }
    return result;
  }

  private static void ConvertResistChannelsData(ResistChannelsData value, WritableMap result) {
    result.putInt("PackNum", value.getPackNum());
    result.putDouble("A1", value.getA1());
    result.putDouble("A2", value.getA2());
    result.putDouble("Bias", value.getBias());
    result.putArray("Values", Arguments.fromArray(value.getValues()));
  }
}