package com.neurotech.filters;

public class Filter {
    static {
        System.loadLibrary("android-filters");
    }

    private long mFilterPtr;
    private boolean mIsFinalized = false;

    public Filter(FilterParam param) {
        mFilterPtr = createFilterByParams(param.getType().NativeInt, param.getSamplingFreq(), param.getCutoffFreq());
    }

    public Filter(String filter) {
        mFilterPtr = createCustomFilter(filter);
    }


    public void reset(){
        reset(mFilterPtr);
    }

    public  void clearParams(){
        clearParams(mFilterPtr);
    }

    public void setParams(String params){
        setParams(mFilterPtr,params);
    }
    public double filter(double x){
        return filter(mFilterPtr, x);
    }

    public double[] filterArray(double[] xs){
        return filterArray(mFilterPtr, xs);
    }
    public  int getId(){
        return getFilterId(mFilterPtr);
    }

    public long getNativePtr() {
        return  mFilterPtr;
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        if(!mIsFinalized){
            mIsFinalized=true;
            deleteFilter(mFilterPtr);
            mFilterPtr = 0;
        }
    }


    private static native long createFilterByParams(int filterType, int samplingFreq, double cutoffFreq);
    private static native long createCustomFilter(String filter);
    private static native void reset(long filterPtr);
    private static native void clearParams(long filterPtr);
    private static native void setParams(long filterPtr,String params);

    private static native double filter(long filterPtr, double x);
    private static native double[] filterArray(long filterPtr, double[] xs);
    private static native int getFilterId(long filterPtr);
    private static native void deleteFilter(long filterPtr);

}
