//
//  BrainBitImpl.m
//  Neurosdk2
//
//  Created by Aseatari on 04.05.2023.
//

#import <Foundation/Foundation.h>
#import "BrainBitImpl.h"
#import "AFullSensorProtected.h"
#import "Constants.h"

@implementation BrainBitImpl

- (instancetype)init:(NSString *)guid
              sensor:(id)bb
           rncontext:(Neurosdk2* _Nonnull)rncontext{
    self = [super init];
    if (self) {
        self.sensor = bb;
        self.guid = guid;
        self.rncontext = rncontext;
        [self initFilters:self.sensor.ChannelsCount];
        [self addCallbacks];
    }
    return self;
}

- (void) addCallbacks{
    [super addCallbacks];
    __weak BrainBitImpl* weakSelf = self;
    [((NTBrainBit*)(self.sensor)) setSignalDataCallbackBrainBit:^(NSArray<NTBrainBitSignalData *> * _Nonnull signals) {
        NSMutableArray* signalsArray = [[NSMutableArray alloc] initWithCapacity: [signals count]];
        @try {
            for (NTBrainBitSignalData* signal in signals) {
                NSNumber* o1 = signal.O1;
                NSNumber* o2 = signal.O2;
                NSNumber* t3 = signal.T3;
                NSNumber* t4 = signal.T4;
                if(self.is66HzEnabled) {
                    double o1RawSample = [signal.O1 doubleValue];
                    o1 = [[NSNumber alloc] initWithDouble:[[self.fLists objectAtIndex:0] filter:o1RawSample]];

                    double o2RawSample = [signal.O2 doubleValue];
                    o2 = [[NSNumber alloc] initWithDouble:[[self.fLists objectAtIndex:1] filter:o2RawSample]];

                    double t3RawSample = [signal.T3 doubleValue];
                    t3 = [[NSNumber alloc] initWithDouble:[[self.fLists objectAtIndex:2] filter:t3RawSample]];

                    double t4RawSample = [signal.T4 doubleValue];
                    t4 = [[NSNumber alloc] initWithDouble:[[self.fLists objectAtIndex:3] filter:t4RawSample]];
                }
                NSDictionary *signalDic = @{ @"Marker" : [NSNumber numberWithInt:signal.Marker],
                                             @"PackNum" : [NSNumber numberWithInt:signal.PackNum],
                                             @"O1" : o1,
                                             @"O2" : o2,
                                             @"T3" : t3,
                                             @"T4" : t4
                };
                [signalsArray addObject:signalDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:signalsArray forKey:DATA_ID];
            NSString* bbsignalEvent = BRAINBIT_SIGNAL_CHANGED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:bbsignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];

    [((NTBrainBit*)(self.sensor)) setResistCallbackBrainBit:^(NTBrainBitResistData * _Nonnull resist) {
        @try {
            NSDictionary *resistDic = @{ @"O1" : resist.O1,
                                         @"O2" : resist.O2,
                                         @"T3" : resist.T3,
                                         @"T4" : resist.T4};
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:resistDic forKey:DATA_ID];
            NSString* bbresistEvent = BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:bbresistEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
}
- (void) removeCallbacks{
    [super removeCallbacks];
    [((NTBrainBit*)(self.sensor)) setSignalDataCallbackBrainBit:nil];
    [((NTBrainBit*)(self.sensor)) setResistCallbackBrainBit:nil];
}

- (void)setGain:(NTSensorGain)gain{
    ((NTBrainBit*)self.sensor).Gain = gain;
}

@end
