//
//  BrainBitImpl.m
//  Neurosdk2
//
//  Created by Aseatari on 04.05.2023.
//

#import <Foundation/Foundation.h>
#import "Headphones2Impl.h"
#import "AFullSensorProtected.h"
#import "Constants.h"

@implementation Headphones2Impl

- (instancetype)init:(NSString *)guid
              sensor:(id)bb
           rncontext:(Neurosdk2* _Nonnull)rncontext{
    self = [super init];
    if (self) {
        self.sensor = bb;
        self.guid = guid;
        self.rncontext = rncontext;
        [self addCallbacks];
    }
    return self;
}

- (void) addCallbacks{
    [super addCallbacks];
    __weak Headphones2Impl* weakSelf = self;
    [((NTHeadphones2*)(self.sensor)) setSignalDataCallback:^(NSArray<NTHeadphones2SignalData *> * _Nonnull signals) {
        @try {
            NSMutableArray* signalsArray = [[NSMutableArray alloc] initWithCapacity: [signals count]];
            for (NTHeadphones2SignalData* signal in signals) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:signal.PackNum],
                    @"Marker" : [NSNumber numberWithInt:signal.Marker],
                    @"Ch1" : signal.Ch1,
                    @"Ch2" : signal.Ch2,
                    @"Ch3" : signal.Ch3,
                    @"Ch4" : signal.Ch4
                };
                [signalsArray addObject:signalDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:signalsArray forKey:DATA_ID];
            NSString* csignalEvent = HEADPHONES2_SIGNAL_RECEIVED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];

    [((NTHeadphones2*)(self.sensor)) setResistCallback:^(NSArray<NTHeadphones2ResistData*>*_Nonnull resists) {
        @try {
            NSMutableArray* resistArray = [[NSMutableArray alloc] initWithCapacity: [resists count]];
            for (NTHeadphones2ResistData* resist in resists) {
                NSDictionary *resistDic = @{
                    @"PackNum" : [NSNumber numberWithInt:resist.PackNum],
                    @"Ch1" : resist.Ch1,
                    @"Ch2" : resist.Ch2,
                    @"Ch3" : resist.Ch3,
                    @"Ch4" : resist.Ch4,
                };
                [resistArray addObject:resistDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:resistArray forKey:DATA_ID];
            NSString* bbresistEvent = HEADPHONES2_RESIST_RECEIVED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:bbresistEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTHeadphones2*)(self.sensor)) setFPGDataCallback:^(NSArray<NTFPGData *> * _Nonnull FPGData) {
        NSMutableArray* FPGDataArray = [[NSMutableArray alloc] initWithCapacity: [FPGData count]];
                for (NTFPGData* data in FPGData) {
                    NSDictionary *dataDic = @{ @"PackNum" : [NSNumber numberWithInt:data.PackNum],
                                               @"IrAmplitude" : data.IrAmplitude,
                                               @"RedAmplitude" : data.RedAmplitude
                    };
                    [FPGDataArray addObject:dataDic];
                }
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:FPGDataArray forKey:DATA_ID];
        NSString* fpgEvent = NEUROSMART_FPGDATA_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent: fpgEvent params:res];
    }];
    [((NTHeadphones2*)(self.sensor)) setMEMSDataCallback:^(NSArray<NTMEMSData *> * _Nonnull MEMSData) {
        NSMutableArray* MEMSDataArray = [[NSMutableArray alloc] initWithCapacity: [MEMSData count]];
        for (NTMEMSData* data in MEMSData) {
            NSDictionary* AccelerometerDic = @{
                @"X" : [NSNumber numberWithDouble: data.Accelerometer.X],
                @"Y" : [NSNumber numberWithDouble: data.Accelerometer.Y],
                @"Z" : [NSNumber numberWithDouble: data.Accelerometer.Z]
            };
            NSDictionary* GyroscopeDic = @{
                @"X" : [NSNumber numberWithDouble: data.Gyroscope.X],
                @"Y" : [NSNumber numberWithDouble: data.Gyroscope.Y],
                @"Z" : [NSNumber numberWithDouble: data.Gyroscope.Z]
            };
            NSDictionary *dataDic = @{
                @"Accelerometer" : AccelerometerDic,
                @"Gyroscope" : GyroscopeDic
            };
            [MEMSDataArray addObject:dataDic];
        }
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:MEMSDataArray forKey:DATA_ID];
        NSString* memsEvent = MEMSDATA_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent:memsEvent params:res];
    }];
    [((NTHeadphones2*)(self.sensor)) setAmpModeCallback:^(NTSensorAmpMode ampMode) {
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:[NSNumber numberWithUnsignedInt:ampMode] forKey:DATA_ID];
        NSString* ampmodeEvent = AMPMODE_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent:ampmodeEvent params:res];
    }];
}
- (void) removeCallbacks{
    [super removeCallbacks];
    [((NTHeadphones2*)(self.sensor)) setResistCallback:nil];
    [((NTHeadphones2*)(self.sensor)) setResistCallback:nil];
    [((NTHeadphones2*)(self.sensor)) setFPGDataCallback:nil];
    [((NTHeadphones2*)(self.sensor)) setMEMSDataCallback:nil];
    [((NTHeadphones2*)(self.sensor)) setAmpModeCallback:nil];
}

- (NSNumber *)getSamplingFrequencyMEMS{
    return @([(NTHeadphones2*)(self.sensor) SamplingFrequencyMEMS]);
}
- (NSNumber *)getSamplingFrequencyFPG{
    return @([(NTHeadphones2*)(self.sensor) SamplingFrequencyFPG]);
}

- (NSNumber *)getSamplingFrequencyResist{
    return @([(NTHeadphones2*)(self.sensor) SamplingFrequencyResist]);
}
- (void)setIrAmplitude:(NTIrAmplitude)amp{
    ((NTHeadphones2*)(self.sensor)).IrAmplitude = amp;
}
- (NSNumber *)getIrAmplitude{
    return @([(NTHeadphones2*)(self.sensor) IrAmplitude]);
}
- (NSNumber *)getRedAmplitude{
    return @([(NTHeadphones2*)(self.sensor) RedAmplitude]);
}
- (void)setRedAmplitude:(NTRedAmplitude)amp{
    ((NTHeadphones2*)(self.sensor)).RedAmplitude = amp;
}
- (NSNumber *)getAmpMode{
    return @([(NTHeadphones2*)(self.sensor) AmpMode]);
}
- (void)setAccSens:(NTSensorAccelerometerSensitivity)accSens{
    ((NTHeadphones2*)self.sensor).AccSens = accSens;
}
- (NSNumber *_Nonnull)getAccSens{
    return @([((NTHeadphones2*)self.sensor) AccSens]);
}
- (void)setGyroSens:(NTSensorGyroscopeSensitivity)gyroSens{
    ((NTHeadphones2*)self.sensor).GyroSens = gyroSens;
}
- (NSNumber *_Nonnull)getGyroSens{
    return @([((NTHeadphones2*)self.sensor) GyroSens]);
}
- (NSDictionary *_Nonnull)getAmplifierParamHP2{
    NTHeadphones2AmplifierParam* param = [(NTHeadphones2*)self.sensor AmplifierParam];
    NSDictionary* result = @{
        @"Current": @(param.Current),
        @"ChSignalUse1": @(param.ChSignalUse1),
        @"ChSignalUse2": @(param.ChSignalUse2),
        @"ChSignalUse3": @(param.ChSignalUse3),
        @"ChSignalUse4": @(param.ChSignalUse4),
        @"ChResistUse1": @(param.ChResistUse1),
        @"ChResistUse2": @(param.ChResistUse2),
        @"ChResistUse3": @(param.ChResistUse3),
        @"ChResistUse4": @(param.ChResistUse4),
        @"ChGain1":@(param.ChGain1),
        @"ChGain2":@(param.ChGain2),
        @"ChGain3":@(param.ChGain3),
        @"ChGain4":@(param.ChGain4),
    };
    return result;
}
- (void)setAmplifierParamHP2:(NTHeadphones2AmplifierParam*_Nonnull)param{
    ((NTHeadphones2*)self.sensor).AmplifierParam = param;
}
- (void)pingNeuroSmart:(NSNumber *)marker{
    [(NTHeadphones2*)(self.sensor) PingNeuroSmart: [marker charValue]];
}

@end
