//
//  NeuroEEGImpl.m
//  Neurosdk2
//
//  Created by Aseatari on 25.10.2023.
//

#import <Foundation/Foundation.h>
#import "NeuroEEGImpl.h"
#import "AFullSensorProtected.h"
#import "Constants.h"

@implementation NeuroEEGImpl

- (instancetype _Nonnull )init:(NSString*_Nonnull)guid
                        sensor:(NTSensor*_Nonnull)sensor
                     rncontext:(Neurosdk2* _Nonnull)rncontext{
    self = [super init];
    if (self) {
        self.sensor = sensor;
        self.guid = guid;
        self.rncontext = rncontext;
        // NTNeuroEEGFSStatus* fsStatus = [(NTNeuroEEG*)self.sensor FSStatus];
        // if (fsStatus.Status == NTSensorFSStatusOK) {
        //     [(NTNeuroEEG*)self.sensor ExecCommand: NTSensorCommandFileSystemStreamClose];
        //     [(NTNeuroEEG*)self.sensor ExecCommand: NTSensorCommandFileSystemDisable];
        // }
        [self addCallbacks];
    }
    return self;
}

- (void) addCallbacks{
    [super addCallbacks];
    
    __weak NeuroEEGImpl* weakSelf = self;
    [((NTNeuroEEG*)self.sensor) setAmpModeCallback:^(NTSensorAmpMode ampMode) {
        @try {
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:[NSNumber numberWithInt:ampMode] forKey:DATA_ID];
            NSString* ampModeEvent = AMPMODE_CHANGED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:ampModeEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTNeuroEEG*)self.sensor) setSignalCallback:^(NSArray<NTSignalChannelsData *> * _Nonnull signals) {
        @try {
            NSMutableArray* signalsArray = [[NSMutableArray alloc] initWithCapacity: [signals count]];
            for (NTSignalChannelsData* signal in signals) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:signal.PackNum],
                    @"Marker" : [NSNumber numberWithInt:signal.Marker],
                    @"Samples" : signal.Samples
                };
                [signalsArray addObject:signalDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:signalsArray forKey:DATA_ID];
            NSString* csignalEvent = NEURRO_EEG_SIGNAL_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTNeuroEEG*)self.sensor) setResistCallback:^(NSArray<NTResistChannelsData *> * _Nonnull resists) {
        @try {
            NSMutableArray* resistArray = [[NSMutableArray alloc] initWithCapacity: [resists count]];
            for (NTResistChannelsData* resist in resists) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:resist.PackNum],
                    @"A1" : [NSNumber numberWithDouble:resist.A1.intValue],
                    @"A2" : [NSNumber numberWithDouble:resist.A2.intValue],
                    @"Bias" : [NSNumber numberWithDouble:resist.Bias.intValue],
                    @"Values" : resist.Values
                };
                [resistArray addObject:signalDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:resistArray forKey:DATA_ID];
            NSString* csignalEvent = NEURRO_EEG_RESIST_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTNeuroEEG*)self.sensor) setSignalResistCallback:^(NSArray<NTSignalChannelsData *> * _Nonnull signals, NSArray<NTResistChannelsData *> * _Nonnull resists) {
        @try {
            NSMutableArray* signalsArray = [[NSMutableArray alloc] initWithCapacity: [signals count]];
            for (NTSignalChannelsData* signal in signals) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:signal.PackNum],
                    @"Marker" : [NSNumber numberWithInt:signal.Marker],
                    @"Samples" : signal.Samples
                };
                [signalsArray addObject:signalDic];
            }
            NSMutableArray* resistsArray = [[NSMutableArray alloc] initWithCapacity: [resists count]];
            for (NTResistChannelsData* resist in resists) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:resist.PackNum],
                    @"A1" : [NSNumber numberWithDouble:resist.A1.intValue],
                    @"A2" : [NSNumber numberWithDouble:resist.A2.intValue],
                    @"Bias" : [NSNumber numberWithDouble:resist.Bias.intValue],
                    @"Values" : resist.Values
                };
                [resistsArray addObject:signalDic];
            }
            NSDictionary *data = @{
                @"signalData" : signalsArray,
                @"resistData" : resistsArray
            };
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:data forKey:DATA_ID];
            NSString* csignalEvent = NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];

    [((NTNeuroEEG*)self.sensor) setSignalRawCallback:^(NSData * _Nonnull signals) {
        @try {
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            NSMutableArray<NSNumber*>* rawSignal = [[NSMutableArray alloc]initWithCapacity:signals.length];
            const void *bytes = [signals bytes];
            for (int i = 0; i < [signals length]; i++)
            {
                [rawSignal addObject:[NSNumber numberWithUnsignedShort:((uint8_t*)bytes)[i]]];
            }
            [res setObject:rawSignal forKey:DATA_ID];
            NSString* csignalEvent = NEURRO_EEG_RAW_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTNeuroEEG*)self.sensor) setFileStreamReadCallback:^(NSArray<NTSensorFileData *> * _Nonnull fileDatas) {
        @try {
            NSMutableArray* dataArray = [[NSMutableArray alloc] initWithCapacity: [fileDatas count]];
            for (NTSensorFileData* fileData in fileDatas) {
                NSMutableArray<NSNumber*>* rawData = [[NSMutableArray alloc]initWithCapacity:fileData.Data.length];
                const void *bytes = [fileData.Data bytes];
                for (int i = 0; i < [fileData.Data length]; i++)
                {
                    [rawData addObject:[NSNumber numberWithUnsignedShort:((uint8_t*)bytes)[i]]];
                }
                
                NSDictionary *dataDic = @{
                    @"OffsetStart" : [NSNumber numberWithUnsignedInt:fileData.OffsetStart],
                    @"DataAmount" : [NSNumber numberWithUnsignedInt:fileData.DataAmount],
                    @"Data": rawData,
                };
                [dataArray addObject:dataDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:dataArray forKey:DATA_ID];
            NSString* csignalEvent = NEURRO_EEG_FILE_STREAM_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];

}

- (void)removeCallbacks{
    [super removeCallbacks];
    [((NTNeuroEEG*)self.sensor) setAmpModeCallback:nil];
    [((NTNeuroEEG*)self.sensor) setSignalCallback:nil];
    [((NTNeuroEEG*)self.sensor) setResistCallback:nil];
    [((NTNeuroEEG*)self.sensor) setSignalResistCallback:nil];

    [((NTNeuroEEG*)self.sensor) setSignalRawCallback:nil];
    [((NTNeuroEEG*)self.sensor) setFileStreamReadCallback:nil];

}

- (NSNumber *_Nonnull)getSurveyId{
    return [((NTNeuroEEG*)self.sensor) SurveyId];
}

- (void)setSurveyId:(NSNumber*_Nonnull)id{
    ((NTNeuroEEG*)self.sensor).SurveyId = id;
}

- (NSNumber *_Nonnull)getAmpMode{
    return @([((NTNeuroEEG*)self.sensor) AmpMode]);
}

- (NSArray *_Nullable)getSupportedChannels{
    NSArray<NTEEGChannelInfo*>* channels = [(NTNeuroEEG*)self.sensor SupportedChannels];
    NSMutableArray* channelsArray = [[NSMutableArray alloc] initWithCapacity: [channels count]];
    for (NTEEGChannelInfo* channel in channels) {
        NSDictionary *dataDic = @{
            @"Id" : [NSNumber numberWithInt:channel.Id],
            @"ChType" : [NSNumber numberWithInt:channel.ChType],
            @"Name" : channel.Name,
            @"Num" : [NSNumber numberWithUnsignedShort:channel.Num],
        };
        [channelsArray addObject:dataDic];
    }
    return [channelsArray copy];
}

- (NSDictionary *_Nonnull)getAmplifierParam{
    NTNeuroEEGAmplifierParam* param = [(NTNeuroEEG*)self.sensor AmplifierParam];
    NSDictionary* result = @{
        @"ReferentResistMesureAllow": @(param.ReferentResistMesureAllow),
        @"Frequency": [NSNumber numberWithInt:param.Frequency],
        @"ReferentMode": [NSNumber numberWithInt:param.ReferentMode],
        @"ChannelMode": param.ChannelMode,
        @"ChannelGain": param.ChannelGain
    };
    return result;
}

- (void)setAmplifierParam:(NTNeuroEEGAmplifierParam*_Nonnull)param{
    ((NTNeuroEEG*)self.sensor).AmplifierParam = param;
}

- (NSNumber *_Nonnull)getSamplingFrequencyResist{
    return @([((NTNeuroEEG*)self.sensor) SamplingFrequencyResist]);
}

- (NSDictionary *_Nonnull)getFSStatus{
    NTNeuroEEGFSStatus* state = [(NTNeuroEEG*)self.sensor FSStatus];
    NSDictionary* result = @{
        @"Status": [NSNumber numberWithInt:state.Status],
        @"IOStatus": [NSNumber numberWithInt:state.IOStatus],
        @"StreamStatus": [NSNumber numberWithInt:state.StreamStatus],
        @"AutosaveSignal": @(state.AutosaveSignal)
    };
    return result;
}

- (NSDictionary *_Nonnull)getFSDiskInfo{
    NTSensorDiskInfo* param = [(NTNeuroEEG*)self.sensor FSDiskInfo];
    NSDictionary* result = @{
        @"TotalSize": [NSNumber numberWithUnsignedLong:param.TotalSize],
        @"FreeSize": [NSNumber numberWithUnsignedLong:param.FreeSize]
    };
    return result;
}

- (NSDictionary*_Nonnull)readFileInfo:(NSString*_Nonnull)fileName{
    NTSensorFileInfo* info = [(NTNeuroEEG*)self.sensor readFileInfoNeuroEEG: fileName];
    NSDictionary* result = @{
        @"FileName": info.FileName,
        @"FileSize": [NSNumber numberWithUnsignedInt:info.FileSize],
        @"ModifiedYear": [NSNumber numberWithUnsignedInt:info.ModifiedYear],
        @"ModifiedMonth": [NSNumber numberWithUnsignedShort:info.ModifiedMonth],
        @"ModifiedDayOfMonth": [NSNumber numberWithUnsignedShort:info.ModifiedDayOfMonth],
        @"ModifiedHour": [NSNumber numberWithUnsignedShort:info.ModifiedHour],
        @"ModifiedMin": [NSNumber numberWithUnsignedShort:info.ModifiedMin],
        @"ModifiedSec": [NSNumber numberWithUnsignedShort:info.ModifiedSec],
        @"Attribute": [NSNumber numberWithUnsignedShort:info.Attribute],
    };
    return result;
}

- (NSArray<NSDictionary*>*_Nonnull)readFileInfoAll:(NSNumber*_Nonnull)maxFiles{
    NSArray<NTSensorFileInfo*> * infos = [(NTNeuroEEG*)self.sensor readFileInfoAllNeuroEEG: maxFiles];
    NSMutableArray* resultArray = [[NSMutableArray alloc] initWithCapacity: [infos count]];
    for (NTSensorFileInfo* info in infos) {
        NSDictionary* infoDic = @{
            @"FileName": info.FileName,
            @"FileSize": [NSNumber numberWithUnsignedInt:info.FileSize],
            @"ModifiedYear": [NSNumber numberWithUnsignedInt:info.ModifiedYear],
            @"ModifiedMonth": [NSNumber numberWithUnsignedShort:info.ModifiedMonth],
            @"ModifiedDayOfMonth": [NSNumber numberWithUnsignedShort:info.ModifiedDayOfMonth],
            @"ModifiedHour": [NSNumber numberWithUnsignedShort:info.ModifiedHour],
            @"ModifiedMin": [NSNumber numberWithUnsignedShort:info.ModifiedMin],
            @"ModifiedSec": [NSNumber numberWithUnsignedShort:info.ModifiedSec],
            @"Attribute": [NSNumber numberWithUnsignedShort:info.Attribute],
        };
        [resultArray addObject:infoDic];
    }
    return resultArray;
}


- (void)writeFile:(NSString*_Nonnull)fileName data:(NSArray<NSNumber *> *_Nonnull)data offset:(NSNumber*_Nonnull)offsetStart{
    NSMutableData* byteData = [NSMutableData new];
    for (NSNumber* number in data) {
        int value = [number charValue];
        [byteData appendBytes:&value length:1];
    }

    [(NTNeuroEEG*)self.sensor writeFileNeuroEEG: fileName data:[byteData copy] offset:offsetStart];
}

- (NSString*_Nonnull)readFile:(NSString*_Nonnull)fileName szData:(NSNumber*_Nonnull)szData offset:(NSNumber*_Nonnull)offsetStart{
    NSData* data = [(NTNeuroEEG*)self.sensor readFileNeuroEEG: fileName szData:szData offset:offsetStart];
    NSString* str = [[NSString alloc] initWithData: data encoding:NSISOLatin1StringEncoding];
    if (str == nil) {
        return @"";
    }
    return str;
}

- (void)deleteFile:(NSString*_Nonnull)fileName{
    [(NTNeuroEEG*)self.sensor deleteFileNeuroEEG: fileName];
}

- (void)deleteAllFiles:(NSString*_Nonnull)fileExt{
    [(NTNeuroEEG*)self.sensor deleteAllFilesNeuroEEG: fileExt];
}

- (NSNumber*_Nonnull)readFileCRC32:(NSString*_Nonnull)fileName totalSize:(NSNumber*_Nonnull)totalSize offsetStart:(NSNumber*_Nonnull)offsetStart{
    return @([(NTNeuroEEG*)self.sensor readFileCRC32NeuroEEG: fileName totalSize:totalSize offsetStart:offsetStart]);
}

- (void)fileStreamAutosave:(NSString*_Nonnull)fileName{
    [(NTNeuroEEG*)self.sensor fileStreamAutosaveNeuroEEG: fileName];
}

- (void)fileStreamRead:(NSString*_Nonnull)fileName totalSize:(NSNumber*_Nonnull)totalSize offset:(NSNumber*_Nonnull)offsetStart{
    [(NTNeuroEEG*)self.sensor fileStreamReadNeuroEEG: fileName totalSize:totalSize offset:offsetStart];
}

@end
