#import "Neurosdk2.h"
#import "Constants.h"

@implementation Neurosdk2{
    ScannerHolder* scannerHolder;
    SensorHolder* sensorHolder;
    
    bool hasListeners;
}
RCT_EXPORT_MODULE()

- (instancetype)init{
    self = [super init];
    if (self) {
        scannerHolder = [ScannerHolder new];
        sensorHolder = [SensorHolder new];
      }
    return self;
}

+(BOOL)requiresMainQueueSetup
{
    return NO;
}

- (NSArray<NSString *> *)supportedEvents
{
    NSString* stateEvent = CONNECTION_CHANGED_EVENT_NAME;
    NSString* batteryEvent = BATTERY_CHANGED_EVENT_NAME;
    
    NSString* electrodeEvent = CALLIBRI_ELECTRODE_STATE_EVENT_NAME;
    NSString* envelopeEvent = CALLIBRI_ENVELOPE_DATA_EVENT_NAME;
    NSString* csignalEvent = CALLIBRI_SIGNAL_DATA_EVENT_NAME;
    NSString* respirationEvent = CALLIBRI_RESPIRATION_DATA_EVENT_NAME;
    NSString* quaterEvent = QUATERNIONDATA_CHANGED_EVENT_NAME;
    
    
    NSString* ampmodeEvent = AMPMODE_CHANGED_EVENT_NAME;
    
    
    NSString* memsEvent = MEMSDATA_CHANGED_EVENT_NAME;
    
    
    NSString* bbsignalEvent = BRAINBIT_SIGNAL_CHANGED_EVENT_NAME;
    NSString* bbresistEvent = BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME;
    
    
    NSString* fpgEvent = NEUROSMART_FPGDATA_CHANGED_EVENT_NAME;
    
    
    NSString* bb2signalEvent = BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME;
    NSString* bb2resistEvent = BRAINBIT2_RESIST_RECEIVED_EVENT_NAME;
    
    
    NSString* hp2signalEvent = HEADPHONES2_SIGNAL_RECEIVED_EVENT_NAME;
    NSString* hp2resistEvent = HEADPHONES2_RESIST_RECEIVED_EVENT_NAME;
    
    
    NSString* nrEegSignalEvent = NEURRO_EEG_SIGNAL_DATA_EVENT_NAME;
    NSString* nrEegResistEvent = NEURRO_EEG_RESIST_DATA_EVENT_NAME;
    NSString* nrEegSignalResistEvent = NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME;
    
    
    NSString* nrEegRawDataEvent = NEURRO_EEG_RAW_DATA_EVENT_NAME;
    NSString* nrEegFileStreamEvent = NEURRO_EEG_FILE_STREAM_DATA_EVENT_NAME;
    
             
    NSString* sensFoundedEvent = SENSOR_LIST_CHANGED_EVENT_NAME;
  return @[stateEvent,
           batteryEvent,
           
           electrodeEvent,
           envelopeEvent,
           csignalEvent,
           respirationEvent,
           quaterEvent,
           
           
           ampmodeEvent,
           
           
           memsEvent,
           
           
           bbsignalEvent,
           bbresistEvent,
           
           
           fpgEvent,
           
           
           nrEegSignalEvent,
           nrEegResistEvent,
           nrEegSignalResistEvent,
           
           
           bb2signalEvent,
           bb2resistEvent,
           
           
           hp2signalEvent,
           hp2resistEvent,
           
           
           nrEegRawDataEvent,
           nrEegFileStreamEvent,
           
           sensFoundedEvent
  ];
}
-(void)startObserving{
    hasListeners = YES;
}

-(void)stopObserving{
    hasListeners = NO;
}

-(void) neuroSendEvent:(NSString*)name params:(id)params{
    if(hasListeners){
        [self sendEventWithName:name body:params];
    }
}

- (NSDictionary*) constantsToExport{
    NSString* stateEvent = CONNECTION_CHANGED_EVENT_NAME;
    NSString* batteryEvent = BATTERY_CHANGED_EVENT_NAME;
    
    NSString* electrodeEvent = CALLIBRI_ELECTRODE_STATE_EVENT_NAME;
    NSString* envelopeEvent = CALLIBRI_ENVELOPE_DATA_EVENT_NAME;
    NSString* csignalEvent = CALLIBRI_SIGNAL_DATA_EVENT_NAME;
    NSString* respirationEvent = CALLIBRI_RESPIRATION_DATA_EVENT_NAME;
    NSString* quaterEvent = QUATERNIONDATA_CHANGED_EVENT_NAME;
    
    
    NSString* ampmodeEvent = AMPMODE_CHANGED_EVENT_NAME;
    
    
    NSString* memsEvent = MEMSDATA_CHANGED_EVENT_NAME;
    
    
    NSString* bbsignalEvent = BRAINBIT_SIGNAL_CHANGED_EVENT_NAME;
    NSString* bbresistEvent = BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME;
    
    
    NSString* fpgEvent = NEUROSMART_FPGDATA_CHANGED_EVENT_NAME;
    
    
    NSString* bb2signalEvent = BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME;
    NSString* bb2resistEvent = BRAINBIT2_RESIST_RECEIVED_EVENT_NAME;
    NSNumber* bb2MaxChCount = [NTBrainBit2 getMaxChCount];
    
    
    NSString* hp2signalEvent = HEADPHONES2_SIGNAL_RECEIVED_EVENT_NAME;
    NSString* hp2resistEvent = HEADPHONES2_RESIST_RECEIVED_EVENT_NAME;
    
    
    NSString* nrEegSignalEvent = NEURRO_EEG_SIGNAL_DATA_EVENT_NAME;
    NSString* nrEegResistEvent = NEURRO_EEG_RESIST_DATA_EVENT_NAME;
    NSString* nrEegSignalResistEvent = NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME;
    NSNumber* nrEegMaxChCount = [NTNeuroEEG getMaxChCount];
    
    
    NSString* nrEegRawDataEvent = NEURRO_EEG_RAW_DATA_EVENT_NAME;
    NSString* nrEegFileStreamEvent = NEURRO_EEG_FILE_STREAM_DATA_EVENT_NAME;
    
             
    NSString* sensFoundedEvent = SENSOR_LIST_CHANGED_EVENT_NAME;
    return @{
              @"CONNECTION_CHANGED_EVENT_NAME" : stateEvent,
              @"BATTERY_CHANGED_EVENT_NAME" : batteryEvent,
            
              @"CALLIBRI_ELECTRODE_STATE_EVENT_NAME" : electrodeEvent,
              @"CALLIBRI_ENVELOPE_DATA_EVENT_NAME" : envelopeEvent,
              @"CALLIBRI_SIGNAL_DATA_EVENT_NAME" : csignalEvent,
              @"CALLIBRI_RESPIRATION_DATA_EVENT_NAME" : respirationEvent,
              @"QUATERNIONDATA_CHANGED_EVENT_NAME" : quaterEvent,
            
            
              @"AMPMODE_CHANGED_EVENT_NAME" : ampmodeEvent,
            
            
              @"MEMSDATA_CHANGED_EVENT_NAME" : memsEvent,
            
            
              @"BRAINBIT_SIGNAL_CHANGED_EVENT_NAME" : bbsignalEvent,
              @"BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME" : bbresistEvent,
            
            
              @"NEUROSMART_FPGDATA_CHANGED_EVENT_NAME" : fpgEvent,
            
            
              @"BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME" : bb2signalEvent,
              @"BRAINBIT2_RESIST_RECEIVED_EVENT_NAME" : bb2resistEvent,
              @"BRAINBIT2_MAX_CHANNELS_COUNT" : bb2MaxChCount,
            
            
              @"NEURRO_EEG_SIGNAL_DATA_EVENT_NAME" : nrEegSignalEvent,
              @"NEURRO_EEG_RESIST_DATA_EVENT_NAME" : nrEegResistEvent,
              @"NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME" : nrEegSignalResistEvent,
              @"NEURRO_EEG_MAX_CHANNELS_COUNT" : nrEegMaxChCount,
            
            
              @"HEADPHONES2_SIGNAL_RECEIVED_EVENT_NAME" : hp2signalEvent,
              @"HEADPHONES2_RESIST_RECEIVED_EVENT_NAME" : hp2resistEvent,
            
            
              @"NEURRO_EEG_RAW_DATA_EVENT_NAME" : nrEegRawDataEvent,
              @"NEURRO_EEG_FILE_STREAM_DATA_EVENT_NAME" : nrEegFileStreamEvent,
            
              @"SENSOR_LIST_CHANGED_EVENT_NAME" : sensFoundedEvent
    };
}

// Don't compile this code when we build for the old architecture.
#ifdef RCT_NEW_ARCH_ENABLED
- (std::shared_ptr<facebook::react::TurboModule>)getTurboModule:
    (const facebook::react::ObjCTurboModule::InitParams &)params
{
    return std::make_shared<facebook::react::NativeNeurosdk2SpecJSI>(params);
}

- (void)closeScanner:(NSString *)guid
             resolve:(RCTPromiseResolveBlock)resolve
              reject:(RCTPromiseRejectBlock)reject {
    [scannerHolder closeScanner:guid resolve:resolve reject:reject];
}

- (void)closeSensor:(NSString *)guid {
    [sensorHolder closeSensor:guid];
}

- (void)connectSensor:(NSString *)guid
              resolve:(RCTPromiseResolveBlock)resolve
               reject:(RCTPromiseRejectBlock)reject {
    [sensorHolder connectSensor:guid resolve:resolve reject:reject];
}

- (void)createScanner:(NSArray *)filters
              resolve:(RCTPromiseResolveBlock)resolve
               reject:(RCTPromiseRejectBlock)reject {
    [scannerHolder createScanner:filters resolve:resolve reject:reject rncontext:self];
}

- (void)createSensor:(NSString *)guid sensorInfo:(JS::NativeNeurosdk2::SensorInfo &)sensorInfo resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    @try {
        NTSensorInfo* info = [[NTSensorInfo alloc] initWithSensFamily:(NTSensorFamily)sensorInfo.SensFamily() sensModel:sensorInfo.SensModel() name:sensorInfo.Name() address:sensorInfo.Address() serialNumber:sensorInfo.SerialNumber() pairingRequared:sensorInfo.PairingRequired() rssi:sensorInfo.RSSI()];
        
        NTScanner* scanner = [scannerHolder getScanner:guid];
        if(scanner == nil){
            reject(@"createSensor", @"Scanner does not exist", nil);
            return;
        }
        
        NSString *uuid = [[NSUUID UUID] UUIDString];
        NTSensor* sensor = [scanner createSensor:info];
        if(sensor == nil){
            reject(@"createSensor", @"Cannot create sensor", nil);
            return;
        }
        
        [sensorHolder addSensor:uuid sensor:sensor rncontext:self];
               
        resolve(uuid);
        
    } @catch (NSException *exception) {
        reject(@"createSensor", exception.description, nil);
    }
}

- (void)disconnectSensor:(NSString *)guid resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    [sensorHolder disconnectSensor:guid resolve:resolve reject:reject];
}

- (void)execCommand:(NSString *)guid command:(double)command resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    [sensorHolder execCommand:guid command:[[NSNumber alloc] initWithDouble:command] resolve:resolve reject:reject];
}

- (NSNumber *)getADCInput:(NSString *)guid {
    return [sensorHolder getADCInput:guid];
}


- (NSNumber *)getAccSens:(NSString *)guid {
    return [sensorHolder getAccSens:guid];
}

- (NSString *)getAddress:(NSString *)guid {
    return [sensorHolder getAddress:guid];
}


- (NSNumber *)getBattPower:(NSString *)guid {
    return [sensorHolder getBattPower:guid];
}

- (NSNumber *)getChannelsCount:(NSString *)guid {
    return [sensorHolder getChannelsCount:guid];
}
- (NSNumber *)getDataOffset:(NSString *)guid {
    return [sensorHolder getDataOffset:guid];
}

- (NSNumber *)getColor:(NSString *)guid {
    return [sensorHolder getColor:guid];
}

- (NSNumber *)getExtSwInput:(NSString *)guid {
    return [sensorHolder getExtSwInput:guid];
}

- (NSNumber *)isSupportedFilter:(NSString *)guid filter:(double)filter {
    return [sensorHolder isSupportedFilter:guid filter:@(filter)];
}

- (NSArray<NSNumber *> *)getSupportedFilters:(NSString *)guid {
    return [sensorHolder getSupportedFilters:guid];
}

- (NSArray<NSNumber *> *)getHardwareFilters:(NSString *)guid {
    return [sensorHolder getHardwareFilters:guid];
}

- (NSNumber *)getFirmwareMode:(NSString *)guid {
    return [sensorHolder getFirmwareMode:guid];
}

- (NSNumber *)getGain:(NSString *)guid {
    return [sensorHolder getGain:guid];
}

- (NSNumber *)getMEMSCalibrateState:(NSString *)guid {
    return [sensorHolder getMEMSCalibrateState:guid];
}

- (NSDictionary *)getMotionAssistantParam:(NSString *)guid {
    return [sensorHolder getMotionAssistantParam:guid];
}

- (NSNumber *)getMotionCounter:(NSString *)guid {
    return [sensorHolder getMotionCounter:guid];
}

- (NSDictionary *)getMotionCounterParam:(NSString *)guid {
    return [sensorHolder getMotionCounterParam:guid];
}

- (NSNumber *)getElectrodeState:(NSString *)guid {
    return [sensorHolder getElectrodeState:guid];
}

- (NSString *)getName:(NSString *)guid {
    return [sensorHolder getName:guid];
}

- (nonnull NSNumber *)get66HzFEnabled:(nonnull NSString *)guid { 
    return [[NSNumber alloc] initWithBool:[sensorHolder get66HzFEnabled:guid]];
}

- (void)set66HzFEnabled:(nonnull NSString *)guid enabled:(BOOL)enabled { 
    [sensorHolder set66HzFEnabled:enabled guid:guid];
}

- (NSNumber *)getRedAmplitude:(NSString *)guid {
    return [sensorHolder getRedAmplitude:guid];
}

- (NSNumber *)getIrAmplitude:(NSString *)guid {
    return [sensorHolder getIrAmplitude:guid];
}


- (NSNumber *)getAmpMode:(NSString *)guid {
    return [sensorHolder getAmpMode:guid];
}

- (NSNumber *)getSamplingFrequencyResist:(NSString *)guid {
    return [sensorHolder getSamplingFrequencyResist:guid];
}

- (NSNumber *)getSamplingFrequency:(NSString *)guid {
    return [sensorHolder getSamplingFrequency:guid];
}

- (NSNumber *)getSamplingFrequencyFPG:(NSString *)guid {
    return [sensorHolder getSamplingFrequencyFPG:guid];
}


- (NSNumber *)getGyroSens:(NSString *)guid {
    return [sensorHolder getGyroSens:guid];
}

- (NSNumber *)getSamplingFrequencyMEMS:(NSString *)guid {
    return [sensorHolder getSamplingFrequencyMEMS:guid];
}
- (NSNumber *)getSamplingFrequencyEnvelope:(NSString *)guid {
    return [sensorHolder getSamplingFrequencyEnvelope:guid];
}

- (NSNumber *)getSensFamily:(NSString *)guid {
    return [sensorHolder getSensFamily:guid];
}

- (void)getSensors:(NSString *)guid resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    [scannerHolder getSensors:guid resolve:resolve reject:reject];
}

- (NSString *)getSerialNumber:(NSString *)guid {
    return [sensorHolder getSerialNumber:guid];
}

- (NSNumber *)getSamplingFrequencyResp:(NSString *)guid {
    return [sensorHolder getSamplingFrequencyResp:guid];
}

- (NSNumber *)getSignalType:(NSString *)guid {
    return [sensorHolder getSignalType:guid];
}

- (NSDictionary *)getStimulatorMAState:(NSString *)guid {
    return [sensorHolder getStimulatorMAState:guid];
}

- (NSDictionary *)getStimulatorParam:(NSString *)guid {
    return [sensorHolder getStimulatorParam:guid];
}

- (void)setADCInput:(NSString *)guid adcInp:(double)adcInp {
    [sensorHolder setADCInput:guid adcInp:(NTSensorADCInput)adcInp];
}

- (void)setDataOffset:(NSString *)guid offset:(double)offset {
    [sensorHolder setDataOffset:guid offset:(NTSensorDataOffset)offset];
}

- (void)setExtSwInput:(NSString *)guid extSwInp:(double)extSwInp {
    [sensorHolder setExtSwInput:guid extSwInp:(NTSensorExternalSwitchInput)extSwInp];
}

- (void)setFirmwareMode:(NSString *)guid mode:(double)mode {
    [sensorHolder setFirmwareMode:guid mode:(NTSensorFirmwareMode)mode];
}

- (void)setGain:(NSString *)guid gain:(double)gain {
    [sensorHolder setGain:guid gain:(NTSensorGain)gain];
}

- (void)setHardwareFilters:(NSString *)guid filters:(NSArray *)filters {
    [sensorHolder setHardwareFilters:guid filters:filters];
}

- (void)setMotionAssistantParam:(NSString *)guid param:(JS::NativeNeurosdk2::CallibriMotionAssistantParams &)param {
    NTCallibriMotionAssistantParams* nativeParams = [[NTCallibriMotionAssistantParams alloc] initWithGyroStart:param.GyroStart() gyroStop:param.GyroStop() limb:(NTCallibriMotionAssistantLimb)param.Limb() minPauseMs:param.MinPauseMs()];
    [sensorHolder setMotionAssistantParam:guid param:nativeParams];
}

- (void)setMotionCounterParam:(NSString *)guid param:(JS::NativeNeurosdk2::CallibriMotionCounterParam &)param {
    NTCallibriMotionCounterParam* nativeParams = [[NTCallibriMotionCounterParam alloc] initWithInsenseThresholdMG:param.InsenseThresholdMG() insenseThresholdSample:param.InsenseThresholdSample()];
    [sensorHolder setMotionCounterParam:guid param:nativeParams];
}

- (void)setSamplingFrequency:(NSString *)guid sf:(double)sf {
    [sensorHolder setSamplingFrequency:guid sf:(NTSensorSamplingFrequency)sf];
}

- (void)setSignalType:(NSString *)guid type:(double)type {
    [sensorHolder setSignalType:guid type:(NTCallibriSignalType)type];
}

- (void)setStimulatorParam:(NSString *)guid param:(JS::NativeNeurosdk2::CallibriStimulationParams &)param {
    NTCallibriStimulationParams* nativeParams = [[NTCallibriStimulationParams alloc] initWithCurrent:param.Current() pulseWidth:param.PulseWidth() frequency:param.Frequency() stimulusDuration:param.StimulusDuration()];

    [sensorHolder setStimulatorParam:guid param:nativeParams];
}

- (NSNumber *)getState:(NSString *)guid {
    return [sensorHolder getState:guid];
}

- (NSDictionary *)getVersion:(NSString *)guid {
    return [sensorHolder getVersion:guid];
}

- (NSNumber *)isSupportedCommand:(NSString *)guid command:(double)command {
    return [sensorHolder isSupportedCommand:guid command:@(command)];
}

- (NSNumber *)isSupportedFeature:(NSString *)guid feature:(double)feature {
    return [sensorHolder isSupportedFeature:guid feature:@(feature)];
}

- (NSNumber *)isSupportedParameter:(NSString *)guid parameter:(double)parameter {
    return [sensorHolder isSupportedParameter:guid parameter:@(parameter)];
}

- (void)pingNeuroSmart:(NSString *)guid marker:(double)marker resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    [sensorHolder pingNeuroSmart:guid marker:@(marker) resolve:resolve reject:reject];
}
- (void)setIrAmplitude:(NSString *)guid amp:(double)amp {
    [sensorHolder setIrAmplitude:guid amp:(NTIrAmplitude)amp];
}
- (void)setRedAmplitude:(NSString *)guid amp:(double)amp {
    [sensorHolder setRedAmplitude:guid amp:(NTRedAmplitude)amp];
}


- (void)setAccSens:(NSString *)guid accSens:(double)accSens {
    [sensorHolder setAccSens:guid accSens:(NTSensorAccelerometerSensitivity)accSens];
}

- (void)setGyroSens:(NSString *)guid gyroSens:(double)gyroSens {
    [sensorHolder setGyroSens:guid gyroSens:(NTSensorGyroscopeSensitivity)gyroSens];
}


- (void)setName:(NSString *)guid name:(NSString *)name {
    [sensorHolder setName:guid name:name];
}

- (void)setSerialNumber:(NSString *)guid sn:(NSString *)sn {
    [sensorHolder setSerialNumber:guid sn:sn];
}

- (void)startScan:(NSString *)guid resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    [scannerHolder startScan:guid resolve:resolve reject:reject];
}

- (void)stopScan:(NSString *)guid resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    [scannerHolder stopScan:guid resolve:resolve reject:reject];
}

- (NSArray<NSNumber *> *)supportedCommands:(NSString *)guid {
    return [sensorHolder supportedCommands:guid];
}

- (NSArray<NSNumber *> *)supportedFeatures:(NSString *)guid {
    return [sensorHolder supportedFeatures:guid];
}

- (NSArray<NSDictionary *> *)supportedParameters:(NSString *)guid {
    return [sensorHolder supportedParameters:guid];
}


- (NSNumber *_Nonnull)getSurveyId:(NSString *_Nonnull)guid {
    return [sensorHolder getSurveyId:guid];
}

- (NSDictionary *_Nonnull)getAmplifierParam:(NSString *_Nonnull)guid {
    return [sensorHolder getAmplifierParam:guid];
}

- (void)setSurveyId:(NSString *_Nonnull)guid id:(double)id {
    [sensorHolder setSurveyId:guid id:[[NSNumber alloc] initWithDouble: id]];
}

- (void)setAmplifierParam:(NSString *_Nonnull)guid param:(JS::NativeNeurosdk2::NeuroEEGAmplifierParam &)param {
    NTNeuroEEGAmplifierParam* nativeParams = [NTNeuroEEGAmplifierParam new];

    NSUInteger maxChCount = [[NTNeuroEEG getMaxChCount] unsignedIntValue];

    nativeParams.ReferentResistMesureAllow = param.ReferentResistMesureAllow();
    nativeParams.Frequency = (NTSensorSamplingFrequency)param.Frequency();
    nativeParams.ReferentMode = (NTEEGRefMode)param.ReferentMode();
    for (int i = 0; i < maxChCount; i++) {
        NSNumber* mode = @(param.ChannelMode()[i]);
        [nativeParams.ChannelMode replaceObjectAtIndex:i withObject:mode];
        
        NSNumber* gain = @(param.ChannelGain()[i]);
        [nativeParams.ChannelGain replaceObjectAtIndex:i withObject:gain];
    }
    
    [sensorHolder setAmplifierParam:guid param:nativeParams];
}


- (NSDictionary *)getAmplifierParamBB2:(NSString *)guid { 
    return [sensorHolder getAmplifierParamBB2:guid];
}

- (NSNumber *)setAmplifierParamBB2:(NSString *)guid param:(JS::NativeNeurosdk2::BrainBit2AmplifierParam &)param { 
    NTBrainBit2AmplifierParam* nativeParam = [NTBrainBit2AmplifierParam new];
    
    NSUInteger maxChCount = [[NTBrainBit2 getMaxChCount] unsignedIntValue];
    
    int trueLength = param.ChGain().size() > maxChCount ? maxChCount : param.ChGain().size();
    if(trueLength > 0){
        nativeParam.ChGain = [[NSMutableArray alloc] initWithCapacity:trueLength];
        for(int i = 0; i < trueLength; i++){
            NSNumber* gain = @(param.ChGain()[i]);
            [nativeParam.ChGain addObject:gain];
        }
    } else {
        nativeParam.ChGain = [[NSMutableArray alloc] initWithCapacity:maxChCount];
        for(int i = 0; i < maxChCount; i++){
            [nativeParam.ChGain addObject:@4];
        }
    }
    
    trueLength = param.ChSignalMode().size() > maxChCount ? maxChCount : param.ChSignalMode().size();
    if(trueLength > 0){
        nativeParam.ChSignalMode = [[NSMutableArray alloc] initWithCapacity:trueLength];
        for(int i = 0; i < trueLength; i++){
            NSNumber* mode = @(param.ChSignalMode()[i]);
            [nativeParam.ChSignalMode addObject:mode];
        }
    } else {
        nativeParam.ChSignalMode = [[NSMutableArray alloc] initWithCapacity:maxChCount];
        for(int i = 0; i < maxChCount; i++){
            [nativeParam.ChSignalMode addObject:@1];
        }
    }
    
    trueLength = param.ChResistUse().size() > maxChCount ? maxChCount : param.ChResistUse().size();
    if(trueLength > 0){
        nativeParam.ChResistUse = [[NSMutableArray alloc] initWithCapacity:trueLength];
        for(int i = 0; i < trueLength; i++){
            NSNumber* rUse = @(param.ChResistUse()[i]);
            [nativeParam.ChResistUse addObject:rUse];
        }
    } else {
        nativeParam.ChResistUse = [[NSMutableArray alloc] initWithCapacity:maxChCount];
        for(int i = 0; i < maxChCount; i++){
            [nativeParam.ChResistUse addObject:@1];
        }
    }
    
    if(param.ChGain().size() < 1 || param.ChSignalMode().size() < 1 || param.ChResistUse().size() < 1){
        nativeParam.Current = NTGenCurrentGenCurr6nA;
    } else {
        nativeParam.Current = (NTGenCurrent)param.Current();
    }
    [sensorHolder setAmplifierParamBB2:guid param:nativeParam];
    return @0;
}


- (NSArray *_Nonnull)getSupportedChannels:(NSString *_Nonnull)guid {
    return [sensorHolder getSupportedChannels:guid];
}


- (NSDictionary *_Nonnull)getFSStatus:(NSString *_Nonnull)guid {
    return [sensorHolder getFSStatus:guid];
}

- (NSDictionary *_Nonnull)getFSDiskInfo:(NSString *_Nonnull)guid {
    return [sensorHolder getFSDiskInfo:guid];
}

- (NSDictionary*_Nonnull)readFileInfo:(NSString *_Nonnull)guid fileName:(NSString*_Nonnull)fileName {
    return [sensorHolder readFileInfo:guid fileName:fileName];
}

- (NSArray<NSDictionary*>*_Nonnull)readFileInfoAll:(NSString *_Nonnull)guid maxFiles:(double)maxFiles {
    return [sensorHolder readFileInfoAll:guid maxFiles:[[NSNumber alloc] initWithDouble:maxFiles]];
}

- (void)writeFile:(NSString *_Nonnull)guid fileName:(NSString*_Nonnull)fileName data:(NSArray<NSNumber *> *)data offset:(double)offsetStart resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    [sensorHolder writeFile:guid fileName:fileName data:data offset:[[NSNumber alloc] initWithDouble:offsetStart] resolve:resolve reject:reject];
}

- (void)readFile:(nonnull NSString *)guid fileName:(nonnull NSString *)fileName sizeData:(double)sizeData offset:(double)offset resolve:(nonnull RCTPromiseResolveBlock)resolve reject:(nonnull RCTPromiseRejectBlock)reject {
    [sensorHolder readFile:guid fileName:fileName szData:[[NSNumber alloc] initWithDouble:sizeData] offset:[[NSNumber alloc] initWithDouble:offset] resolve:resolve reject:reject];
}

- (void)deleteFile:(NSString *_Nonnull)guid fileName:(NSString*_Nonnull)fileName resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject{
    [sensorHolder deleteFile:guid fileName:fileName resolve:resolve reject:reject];
}

- (void)deleteAllFiles:(NSString *_Nonnull)guid fileExt:(NSString*_Nonnull)fileExt resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject{
    [sensorHolder deleteAllFiles:guid fileExt:fileExt resolve:resolve reject:reject];
}

- (nonnull NSNumber *)readFileCRC32:(nonnull NSString *)guid fileName:(nonnull NSString *)fileName totalSize:(double)totalSize offset:(double)offset {
    return [sensorHolder readFileCRC32:guid fileName:fileName totalSize:[[NSNumber alloc] initWithDouble:totalSize] offsetStart:[[NSNumber alloc] initWithDouble:offset]];
}

- (void)fileStreamAutosave:(NSString *_Nonnull)guid fileName:(NSString*_Nonnull)fileName {
    [sensorHolder fileStreamAutosave:guid fileName:fileName];
}

- (void)fileStreamRead:(NSString *_Nonnull)guid fileName:(NSString*_Nonnull)fileName totalSize:(double)totalSize offset:(double)offsetStart {
    [sensorHolder fileStreamRead:guid fileName:fileName totalSize:[[NSNumber alloc] initWithDouble:totalSize] offset:[[NSNumber alloc] initWithDouble:offsetStart]];
}

- (void)startProcessingData:(JS::NativeNeurosdk2::NeuroEEGAmplifierParam &)param {
    NTNeuroEEGAmplifierParam* nativeParams = [NTNeuroEEGAmplifierParam new];

    NSUInteger maxChCount = [[NTNeuroEEG getMaxChCount] unsignedIntValue];

    nativeParams.ReferentResistMesureAllow = param.ReferentResistMesureAllow();
    nativeParams.Frequency = (NTSensorSamplingFrequency)param.Frequency();
    nativeParams.ReferentMode = (NTEEGRefMode)param.ReferentMode();
    for (int i = 0; i < maxChCount; i++) {
        NSNumber* mode = @(param.ChannelMode()[i]);
        [nativeParams.ChannelMode replaceObjectAtIndex:i withObject:mode];
        
        NSNumber* gain = @(param.ChannelGain()[i]);
        [nativeParams.ChannelGain replaceObjectAtIndex:i withObject:gain];
    }
    [sensorHolder startProcessingData:nativeParams];
}

- (void)parseRawSignal:(NSArray<NSNumber *> *)data resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject {
    [sensorHolder parseRawSignal:data resolve:resolve reject:reject];
}

- (void)stopProcessingData{
    [sensorHolder stopProcessingData];
}


- (NSDictionary *)getAmplifierParamHP2:(NSString *)guid { 
    return [sensorHolder getAmplifierParamHP2:guid];
}

- (void)setAmplifierParamHP2:(NSString *)guid param:(JS::NativeNeurosdk2::Headphones2AmplifierParam &)param { 
        NTHeadphones2AmplifierParam* nativeParam = [[NTHeadphones2AmplifierParam alloc] initWithChSignalUse1:param.ChSignalUse1()
                                                                                            chSignalUse2:param.ChSignalUse2()
                                                                                            chSignalUse3:param.ChSignalUse3()
                                                                                            chSignalUse4:param.ChSignalUse4()
                                                                                            chResistUse1:param.ChResistUse1()
                                                                                            chResistUse2:param.ChResistUse2()
                                                                                            chResistUse3:param.ChResistUse3()
                                                                                            chResistUse4:param.ChResistUse4()
                                                                                                 chGain1:(NTSensorGain)param.ChGain1()
                                                                                                 chGain2:(NTSensorGain)param.ChGain2()
                                                                                                 chGain3:(NTSensorGain)param.ChGain3()
                                                                                                 chGain4:(NTSensorGain)param.ChGain4()
                                                                                                 current:(NTGenCurrent)param.Current()];
    
    [sensorHolder setAmplifierParamHP2:guid param:nativeParam];
}



#else

RCT_REMAP_METHOD(closeScanner,
                   closeScanner:(NSString *)guid
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    [scannerHolder closeScanner:guid resolve:resolve reject:reject];
}

RCT_REMAP_METHOD(closeSensor,
                   closeSensor:(NSString *)guid) {
    [sensorHolder closeSensor:guid];
}

RCT_REMAP_METHOD(connectSensor,
                   connectSensor:(NSString *)guid
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    [sensorHolder connectSensor:guid resolve:resolve reject:reject];
}

RCT_REMAP_METHOD(createScanner,
                   createScanner:(NSArray *)filters
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    [scannerHolder createScanner:filters resolve:resolve reject:reject rncontext:self];
}

RCT_REMAP_METHOD(createSensor,
                   createSensor:(NSString *)guid
                   sensorInfo:(NSDictionary*)sensorInfo
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    @try {
        NTSensorInfo* info = [[NTSensorInfo alloc] initWithSensFamily: (NTSensorFamily)[sensorInfo[@"SensFamily"] intValue]
                                                               sensModel:[sensorInfo[@"SensModel"] intValue]
                                                                    name:sensorInfo[@"Name"]
                                                                 address:sensorInfo[@"Address"]
                                                            serialNumber:sensorInfo[@"SerialNumber"]
                                                         pairingRequared:[sensorInfo[@"PairingRequired"] boolValue]
                                                         rssi:[sensorInfo[@"RSSI"] shortValue]
                ];
        
        NTScanner* scanner = [scannerHolder getScanner:guid];
        if(scanner == nil){
            reject(@"createSensor", @"Scanner does not exist", nil);
            return;
        }
        
        NSString *uuid = [[NSUUID UUID] UUIDString];
        NTSensor* sensor = [scanner createSensor:info];
        if(sensor == nil){
            reject(@"createSensor", @"Cannot create sensor", nil);
            return;
        }
        
        [sensorHolder addSensor:uuid sensor:sensor rncontext:self];
               
        resolve(uuid);
        
    } @catch (NSException *exception) {
        reject(@"createSensor", exception.description, nil);
    }
}

RCT_REMAP_METHOD(disconnectSensor,
                   disconnectSensor:(NSString *)guid
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    [sensorHolder disconnectSensor:guid resolve:resolve reject:reject];
}

RCT_REMAP_METHOD(execCommand,
                   execCommand:(NSString *)guid
                   command:(NSNumber*_Nonnull)command
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    [sensorHolder execCommand:guid command:command resolve:resolve reject:reject];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getADCInput, NSNumber *_Nonnull,
                                        getADCInput:(NSString *)guid) {
    return [sensorHolder getADCInput:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getColor, NSNumber *_Nonnull,
getColor:(NSString *)guid) {
    return [sensorHolder getColor:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getExtSwInput, NSNumber *_Nonnull,
getExtSwInput:(NSString *)guid) {
    return [sensorHolder getExtSwInput:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(isSupportedFilter, NSNumber *_Nonnull,
isSupportedFilter:(NSString *)guid filter:(NSNumber*_Nonnull)filter) {
    return [sensorHolder isSupportedFilter:guid filter:filter];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSupportedFilters, NSArray<NSNumber *> *_Nonnull,
getSupportedFilters:(NSString *)guid) {
    return [sensorHolder getSupportedFilters:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getHardwareFilters, NSArray<NSNumber *> *,
getHardwareFilters:(NSString *)guid) {
    return [sensorHolder getHardwareFilters:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getMEMSCalibrateState, NSNumber *_Nonnull,
getMEMSCalibrateState:(NSString *)guid) {
    return [sensorHolder getMEMSCalibrateState:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getMotionAssistantParam, NSDictionary *,
getMotionAssistantParam:(NSString *)guid) {
    return [sensorHolder getMotionAssistantParam:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getMotionCounter, NSNumber *_Nonnull,
getMotionCounter:(NSString *)guid) {
    return [sensorHolder getMotionCounter:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getMotionCounterParam, NSDictionary *,
getMotionCounterParam:(NSString *)guid) {
    return [sensorHolder getMotionCounterParam:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getElectrodeState, NSNumber *_Nonnull,
getElectrodeState:(NSString *)guid) {
    return [sensorHolder getElectrodeState:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSamplingFrequencyResp, NSNumber *_Nonnull,
getSamplingFrequencyResp:(NSString *)guid) {
    return [sensorHolder getSamplingFrequencyResp:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSignalType, NSNumber *_Nonnull,
getSignalType:(NSString *)guid) {
    return [sensorHolder getSignalType:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getStimulatorMAState, NSDictionary *,
getStimulatorMAState:(NSString *)guid) {
    return [sensorHolder getStimulatorMAState:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getStimulatorParam, NSDictionary *,
getStimulatorParam:(NSString *)guid) {
    return [sensorHolder getStimulatorParam:guid];
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getAccSens, NSNumber *_Nonnull,
                                        getAccSens:(NSString *)guid) {
    return [sensorHolder getAccSens:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getGyroSens, NSNumber *_Nonnull,
getGyroSens:(NSString *)guid) {
    return [sensorHolder getGyroSens:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSamplingFrequencyMEMS, NSNumber *_Nonnull,
getSamplingFrequencyMEMS:(NSString *)guid) {
    return [sensorHolder getSamplingFrequencyMEMS:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSamplingFrequencyEnvelope, NSNumber *_Nonnull,
getSamplingFrequencyEnvelope:(NSString *)guid) {
    return [sensorHolder getSamplingFrequencyEnvelope:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setAccSens, id,
        setAccSens:(NSString *)guid
        accSens:(NSNumber *_Nonnull)accSens) {
    [sensorHolder setAccSens:guid accSens:(NTSensorAccelerometerSensitivity)[accSens intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getAddress, NSString *,
                                        getAddress:(NSString *)guid) {
    return [sensorHolder getAddress:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getAmpMode, NSNumber *_Nonnull,
                                        getAmpMode:(NSString *)guid) {
    return [sensorHolder getAmpMode:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSamplingFrequencyResist, NSNumber *_Nonnull,
                                        getSamplingFrequencyResist:(NSString *)guid) {
    return [sensorHolder getSamplingFrequencyResist:guid];
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getIrAmplitude, NSNumber *_Nonnull,
getIrAmplitude:(NSString *)guid) {
    return [sensorHolder getIrAmplitude:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getRedAmplitude, NSNumber *_Nonnull,
getRedAmplitude:(NSString *)guid) {
    return [sensorHolder getRedAmplitude:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSamplingFrequencyFPG, NSNumber *_Nonnull,
getSamplingFrequencyFPG:(NSString *)guid) {
    return [sensorHolder getSamplingFrequencyFPG:guid];
}

RCT_REMAP_METHOD(pingNeuroSmart,
        pingNeuroSmart:(NSString *)guid
        marker:(NSNumber*_Nonnull)marker
        resolve:(RCTPromiseResolveBlock)resolve
        reject:(RCTPromiseRejectBlock)reject) {
    [sensorHolder pingNeuroSmart:guid marker:marker resolve:resolve reject:reject];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setIrAmplitude, id,
        setIrAmplitude:(NSString *)guid
        amp:(NSNumber *_Nonnull)amp) {
    [sensorHolder setIrAmplitude:guid amp:(NTIrAmplitude)[amp intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getBattPower, NSNumber *_Nonnull,
                                        getBattPower:(NSString *)guid) {
    return [sensorHolder getBattPower:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getChannelsCount, NSNumber *_Nonnull,
                                        getChannelsCount:(NSString *)guid) {
    return [sensorHolder getChannelsCount:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getDataOffset, NSNumber *_Nonnull,
                                        getDataOffset:(NSString *)guid) {
    return [sensorHolder getDataOffset:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getFirmwareMode, NSNumber *_Nonnull,
                                        getFirmwareMode:(NSString *)guid) {
    return [sensorHolder getFirmwareMode:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getGain, NSNumber *_Nonnull,
                                        getGain:(NSString *)guid) {
    return [sensorHolder getGain:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getName, NSString *,
                                        getName:(NSString *)guid) {
    return [sensorHolder getName:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSamplingFrequency, NSNumber *_Nonnull,
                                        getSamplingFrequency:(NSString *)guid) {
    return [sensorHolder getSamplingFrequency:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSensFamily, NSNumber *_Nonnull,
                                        getSensFamily:(NSString *)guid) {
    return [sensorHolder getSensFamily:guid];
}

RCT_REMAP_METHOD(getSensors,
                   getSensors:(NSString *)guid
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    [scannerHolder getSensors:guid resolve:resolve reject:reject];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSerialNumber, NSString *,
                                        getSerialNumber:(NSString *)guid) {
    return [sensorHolder getSerialNumber:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getState, NSNumber *_Nonnull,
                                        getState:(NSString *)guid) {
    return [sensorHolder getState:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getVersion, NSDictionary *,
                                        getVersion:(NSString *)guid) {
    return [sensorHolder getVersion:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(get66HzFEnabled, NSNumber *,
                                      get66HzFEnabled:(NSString *)guid){
    return [[NSNumber alloc] initWithBool:[sensorHolder get66HzFEnabled:guid]];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(set66HzFEnabled, id,
                                      set66HzFEnabled:(NSString *)guid
                                      enabled:(BOOL)enabled){
    [sensorHolder set66HzFEnabled:enabled guid:guid];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(isSupportedCommand, NSNumber *_Nonnull,
                                        isSupportedCommand:(NSString *)guid
                                        command:(NSNumber *_Nonnull)command) {
    return [sensorHolder isSupportedCommand:guid command:command];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(isSupportedFeature, NSNumber *_Nonnull,
                                        isSupportedFeature:(NSString *)guid
                                        feature:(NSNumber*_Nonnull)feature) {
    return [sensorHolder isSupportedFeature:guid feature:feature];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(isSupportedParameter, NSNumber *_Nonnull,
                                        isSupportedParameter:(NSString *)guid
                                        parameter:(NSNumber*_Nonnull)parameter) {
    return [sensorHolder isSupportedParameter:guid parameter:parameter];
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setADCInput, id,
                                        setADCInput:(NSString *)guid
                                        adcInp:(NSNumber*_Nonnull)adcInp) {
    [sensorHolder setADCInput:guid adcInp:(NTSensorADCInput)[adcInp intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setDataOffset, id,
                                        setDataOffset:(NSString *)guid
                                        offset:(NSNumber *_Nonnull)offset) {
    [sensorHolder setDataOffset:guid offset:(NTSensorDataOffset)[offset intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setExtSwInput, id,
                                        setExtSwInput:(NSString *)guid
                                        extSwInp:(NSNumber *_Nonnull)extSwInp) {
    [sensorHolder setExtSwInput:guid extSwInp:(NTSensorExternalSwitchInput)[extSwInp intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setFirmwareMode, id,
                                        setFirmwareMode:(NSString *)guid
                                        mode:(NSNumber *_Nonnull)mode) {
    [sensorHolder setFirmwareMode:guid mode:(NTSensorFirmwareMode)[mode intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setGain, id,
                                        setGain:(NSString *)guid
                                        gain:(NSNumber *_Nonnull)gain) {
    [sensorHolder setGain:guid gain:(NTSensorGain)[gain intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setHardwareFilters, id,
        setHardwareFilters:(NSString *)guid
        filters:(NSArray *)filters) {
    [sensorHolder setHardwareFilters:guid filters:filters];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setMotionAssistantParam, id,
        setMotionAssistantParam:(NSString *)guid
        param:(NSDictionary*)param) {
    NTCallibriMotionAssistantParams* assistantParams = [[NTCallibriMotionAssistantParams alloc] initWithGyroStart:[param[@"GyroStart"] charValue]
                                                                                                         gyroStop:[param[@"GyroStop"] charValue]
    limb:(NTCallibriMotionAssistantLimb)[param[@"Limb"] intValue]
    minPauseMs: [param[@"MinPauseMs"] charValue]
    ];
    [sensorHolder setMotionAssistantParam:guid param:assistantParams];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setMotionCounterParam, id,
        setMotionCounterParam:(NSString *)guid
        param:(NSDictionary*)param) {
    NTCallibriMotionCounterParam* counterParams = [[NTCallibriMotionCounterParam alloc] initWithInsenseThresholdMG:[param[@"InsenseThresholdMG"] shortValue]
                                                                                            insenseThresholdSample:[param[@"InsenseThresholdSample"] shortValue]
    ];
    [sensorHolder setMotionCounterParam:guid param:counterParams];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setSamplingFrequency, id,
        setSamplingFrequency:(NSString *)guid
        sf:(NSNumber *_Nonnull)sf) {
    [sensorHolder setSamplingFrequency:guid sf:(NTSensorSamplingFrequency)[sf intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setSignalType, id,
        setSignalType:(NSString *)guid
        type:(NSNumber *_Nonnull)type) {
    [sensorHolder setSignalType:guid type:(NTCallibriSignalType)[type intValue]];
    return nil;
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setGyroSens, id,
                                        setGyroSens:(NSString *)guid
                                        gyroSens:(NSNumber *_Nonnull)gyroSens) {
    [sensorHolder setGyroSens:guid gyroSens:(NTSensorGyroscopeSensitivity)[gyroSens intValue]];
    return nil;
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setName, id,
                                        setName:(NSString *)guid
                                        name:(NSString *)name) {
    [sensorHolder setName:guid name:name];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setRedAmplitude, id,
                                        setRedAmplitude:(NSString *)guid
                                        amp:(NSNumber *_Nonnull)amp) {
    [sensorHolder setRedAmplitude:guid amp:(NTRedAmplitude)[amp intValue]];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setSerialNumber, id,
                                        setSerialNumber:(NSString *)guid
                                        sn:(NSString *)sn) {
    [sensorHolder setSerialNumber:guid sn:sn];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setStimulatorParam, id,
                                        setStimulatorParam:(NSString *)guid
                                        param:(NSDictionary*_Nonnull)param) {
    NTCallibriStimulationParams* stimulationParams = [[NTCallibriStimulationParams alloc] initWithCurrent:[param[@"Current"] shortValue]
            pulseWidth:[param[@"PulseWidth"] shortValue]
            frequency:[param[@"Frequency"] charValue]
            stimulusDuration: [param[@"StimulusDuration"] shortValue]
        ];
    [sensorHolder setStimulatorParam:guid param:stimulationParams];
    return nil;
}

RCT_REMAP_METHOD(startScan,
                   startScan:(NSString *)guid
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    [scannerHolder startScan:guid resolve:resolve reject:reject];
}

RCT_REMAP_METHOD(stopScan,
                   stopScan:(NSString *)guid
                   resolve:(RCTPromiseResolveBlock)resolve
                   reject:(RCTPromiseRejectBlock)reject) {
    [scannerHolder stopScan:guid resolve:resolve reject:reject];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(supportedCommands, NSArray<NSNumber *> *,
                                        supportedCommands:(NSString *)guid) {
    return [sensorHolder supportedCommands:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(supportedFeatures, NSArray<NSNumber *> *,
                                        supportedFeatures:(NSString *)guid) {
    return [sensorHolder supportedFeatures:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(supportedParameters, NSArray<NSDictionary *> *,
                                        supportedParameters:(NSString *)guid) {
    return  [sensorHolder supportedParameters:guid];
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSurveyId, NSNumber *_Nonnull,
                                        getSurveyId:(NSString *)guid) {
    return [sensorHolder getSurveyId:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getAmplifierParam, NSDictionary *_Nonnull,
                                        getAmplifierParam:(NSString *)guid) {
    return [sensorHolder getAmplifierParam:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setSurveyId, id,
                   setSurveyId:(NSString *)guid
                   id:(NSNumber *_Nonnull)id) {
    [sensorHolder setSurveyId:guid id:id];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setAmplifierParam, id,
                   setAmplifierParam:(NSString *)guid
                   param:(NSDictionary*_Nonnull)param) {
    NTNeuroEEGAmplifierParam* nativeParams = [NTNeuroEEGAmplifierParam new];

    NSUInteger maxChCount = [[NTNeuroEEG getMaxChCount] unsignedIntValue];

    nativeParams.ReferentResistMesureAllow = [param[@"ReferentResistMesureAllow"] intValue];
    nativeParams.Frequency = (NTSensorSamplingFrequency)[param[@"Frequency"] intValue];
    nativeParams.ReferentMode = (NTEEGRefMode)[param[@"ReferentMode"] intValue];
    NSArray<NSNumber *>* chModeInput = [[NSArray alloc] initWithArray:param[@"ChannelMode"]];
    NSArray<NSNumber *>* chGainInput = [[NSArray alloc] initWithArray:param[@"ChannelGain"]];
    for (int i = 0; i < maxChCount; i++) {
        [nativeParams.ChannelMode replaceObjectAtIndex:i withObject:chModeInput[i]];
        [nativeParams.ChannelGain replaceObjectAtIndex:i withObject:chGainInput[i]];
    }

    [sensorHolder setAmplifierParam:guid param:nativeParams];
    return nil;
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getSupportedChannels, NSArray *_Nullable,
                                        getSupportedChannels:(NSString *)guid) {
    return [sensorHolder getSupportedChannels:guid];
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getAmplifierParamBB2, NSDictionary *_Nonnull,
                                        getAmplifierParamBB2:(NSString *)guid) {
    return [sensorHolder getAmplifierParamBB2:guid];
}
RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setAmplifierParamBB2, NSNumber *_Nonnull,
                                        setAmplifierParamBB2:(NSString *)guid
                                      param:(NSDictionary*_Nonnull)param) {
    NTBrainBit2AmplifierParam* nativeParam = [NTBrainBit2AmplifierParam new];
    
    NSUInteger maxChCount = [[NTBrainBit2 getMaxChCount] unsignedIntValue];
    
    if([param objectForKey:@"Current"] != nil){
        nativeParam.Current = (NTGenCurrent)[param[@"Current"] intValue];
    }
    else {
        nativeParam.Current = NTGenCurrentGenCurr6nA;
    }
    
    if([param objectForKey:@"ChSignalMode"] != nil){
        nativeParam.ChSignalMode = [[NSMutableArray alloc] initWithArray:param[@"ChSignalMode"]];
    } else {
        nativeParam.ChSignalMode = [[NSMutableArray alloc] initWithCapacity:maxChCount];
        for (NSUInteger i = 0; i < maxChCount; i++) {
            [nativeParam.ChSignalMode addObject:@1];
        }
    }
    
    if([param objectForKey:@"ChGain"] != nil){
        nativeParam.ChGain = [[NSMutableArray alloc] initWithArray:param[@"ChGain"]];
    } else {
        nativeParam.ChGain = [[NSMutableArray alloc] initWithCapacity:maxChCount];
        for (NSUInteger i = 0; i < maxChCount; i++) {
            [nativeParam.ChGain addObject:@4];
        }
    }
    
    if([param objectForKey:@"ChResistUse"] != nil){
        nativeParam.ChResistUse = [[NSMutableArray alloc] initWithArray:param[@"ChResistUse"]];
    } else {
        nativeParam.ChResistUse = [[NSMutableArray alloc] initWithCapacity:maxChCount];
        for (NSUInteger i = 0; i < maxChCount; i++) {
            [nativeParam.ChResistUse addObject:@1];
        }
    }
    [sensorHolder setAmplifierParamBB2:guid param:nativeParam];
    
    return @0;
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getAmplifierParamHP2, NSDictionary *_Nonnull,
                                        getAmplifierParamHP2:(NSString *)guid) {
    return [sensorHolder getAmplifierParamHP2:guid];
}
RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(setAmplifierParamHP2, NSDictionary *_Nonnull,
                                        setAmplifierParamHP2:(NSString *)guid
                                        param:(NSDictionary*_Nonnull)param) {
    NTHeadphones2AmplifierParam* nativeParam = [[NTHeadphones2AmplifierParam alloc] initWithChSignalUse1:[param[@"ChSignalUse1"] boolValue]
                                                                                            chSignalUse2:[param[@"ChSignalUse2"] boolValue]
                                                                                            chSignalUse3:[param[@"ChSignalUse3"] boolValue]
                                                                                            chSignalUse4:[param[@"ChSignalUse4"] boolValue]
                                                                                            chResistUse1:[param[@"ChResistUse1"] boolValue]
                                                                                            chResistUse2:[param[@"ChResistUse2"] boolValue]
                                                                                            chResistUse3:[param[@"ChResistUse3"] boolValue]
                                                                                            chResistUse4:[param[@"ChResistUse4"] boolValue]
                                                                                                 chGain1:(NTSensorGain)[param[@"ChGain1"] intValue]
                                                                                                 chGain2:(NTSensorGain)[param[@"ChGain2"] intValue]
                                                                                                 chGain3:(NTSensorGain)[param[@"ChGain3"] intValue]
                                                                                                 chGain4:(NTSensorGain)[param[@"ChGain4"] intValue]
                                                                                                 current:(NTGenCurrent)[param[@"Current"] intValue]];
    
    [sensorHolder setAmplifierParamHP2:guid param:nativeParam];
    
    return nil;
}


RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getFSStatus, NSDictionary *_Nonnull,
                                        getFSStatus:(NSString *)guid) {
    return [sensorHolder getFSStatus:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(getFSDiskInfo, NSDictionary *_Nonnull,
                                        getFSDiskInfo:(NSString *)guid) {
    return [sensorHolder getFSDiskInfo:guid];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(readFileInfo, NSDictionary*_Nonnull,
                   readFileInfo:(NSString *)guid
                   fileName:(NSString*_Nonnull)fileName) {
    return [sensorHolder readFileInfo:guid fileName:fileName];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(readFileInfoAll, NSArray<NSDictionary*>*_Nonnull,
                   readFileInfoAll:(NSString *)guid
                   maxFiles:(NSNumber*_Nonnull)maxFiles) {
    return [sensorHolder readFileInfoAll:guid maxFiles:maxFiles];
}

RCT_REMAP_METHOD(writeFile, guid:(NSString *)guid
                            fileName:(NSString*_Nonnull)fileName
                            data:(NSArray<NSNumber *> *)data
                            offset:(NSNumber*_Nonnull)offsetStart
                            resolve:(RCTPromiseResolveBlock)resolve
                            reject:(RCTPromiseRejectBlock)reject) {
    [sensorHolder writeFile:guid fileName:fileName data:data offset:offsetStart resolve:resolve reject:reject];
}

RCT_REMAP_METHOD(readFile,
                 guid:(NSString *)guid
                 fileName:(NSString*_Nonnull)fileName
                 szData:(NSNumber*_Nonnull)szData
                 offset:(NSNumber*_Nonnull)offsetStart
                 resolve:(RCTPromiseResolveBlock)resolve
                 reject:(RCTPromiseRejectBlock)reject) {
    [sensorHolder readFile:guid fileName:fileName szData:szData offset:offsetStart resolve:resolve reject:reject];
}

RCT_REMAP_METHOD(deleteFile,
                 guid:(NSString *)guid
                 fileName:(NSString*_Nonnull)fileName
                 resolve:(RCTPromiseResolveBlock)resolve
                 reject:(RCTPromiseRejectBlock)reject) {
    [sensorHolder deleteFile:guid fileName:fileName resolve:resolve reject:reject];
}

RCT_REMAP_METHOD(deleteAllFiles,
                 guid:(NSString *)guid
                 fileExt:(NSString*_Nonnull)fileExt
                 resolve:(RCTPromiseResolveBlock)resolve
                 reject:(RCTPromiseRejectBlock)reject) {
    [sensorHolder deleteAllFiles:guid fileExt:fileExt resolve:resolve reject:reject];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(readFileCRC32, NSNumber*_Nonnull,
                   readFileCRC32:(NSString *)guid
                   fileName:(NSString*_Nonnull)fileName
                   totalSize:(NSNumber*_Nonnull)totalSize
                   offsetStart:(NSNumber*_Nonnull)offsetStart) {
    return [sensorHolder readFileCRC32:guid fileName:fileName totalSize:totalSize offsetStart:offsetStart];
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(fileStreamAutosave, id,
                   fileStreamAutosave:(NSString *)guid
                   fileName:(NSString*_Nonnull)fileName) {
    [sensorHolder fileStreamAutosave:guid fileName:fileName];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(fileStreamRead, id,
                   fileStreamRead:(NSString *)guid
                   fileName:(NSString*_Nonnull)fileName
                   totalSize:(NSNumber*_Nonnull)totalSize
                   offset:(NSNumber*_Nonnull)offsetStart) {
    [sensorHolder fileStreamRead:guid fileName:fileName totalSize:totalSize offset:offsetStart];
    return nil;
}

RCT_REMAP_BLOCKING_SYNCHRONOUS_METHOD(startProcessingData, id,
                                      startProcessingData:(NSDictionary*_Nonnull)param){
    NTNeuroEEGAmplifierParam* nativeParams = [NTNeuroEEGAmplifierParam new];

    NSUInteger maxChCount = [[NTNeuroEEG getMaxChCount] unsignedIntValue];

    nativeParams.ReferentResistMesureAllow = [param[@"ReferentResistMesureAllow"] intValue];
    nativeParams.Frequency = (NTSensorSamplingFrequency)[param[@"Frequency"] intValue];
    nativeParams.ReferentMode = (NTEEGRefMode)[param[@"ReferentMode"] intValue];
    NSArray<NSNumber *>* chModeInput = [[NSArray alloc] initWithArray:param[@"ChannelMode"]];
    NSArray<NSNumber *>* chGainInput = [[NSArray alloc] initWithArray:param[@"ChannelGain"]];
    for (int i = 0; i < maxChCount; i++) {
        [nativeParams.ChannelMode replaceObjectAtIndex:i withObject:chModeInput[i]];
        [nativeParams.ChannelGain replaceObjectAtIndex:i withObject:chGainInput[i]];
    }
    [sensorHolder startProcessingData:nativeParams];
    return nil;
}

RCT_REMAP_METHOD(parseRawSignal,
                 data:(NSArray<NSNumber *> *)data
                 resolve:(RCTPromiseResolveBlock)resolve
                 reject:(RCTPromiseRejectBlock)reject){
    [sensorHolder parseRawSignal:data resolve:resolve reject:reject];
}

RCT_REMAP_METHOD(stopProcessingData, id){
    [sensorHolder stopProcessingData];
}


#endif

@end
